/*
 * Copyright (c) 2003 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#ifndef _OCHUSHA_UI_H_
#define _OCHUSHA_UI_H_

#include "ochusha.h"
#include "ochusha_private.h"

#include <glib.h>
#include <gtk/gtk.h>

#define DEBUG_GUI		0
#define DEBUG_GUI_MOST		0
#define DEBUG_THREAD		0
#define DEBUG_THREAD_MOST	0
#define DEBUG_NETWORK		0
#define DEBUG_NETWORK_MOST	0
#define DEBUG_WIDGET		0
#define DEBUG_WIDGET_MOST	0

#define ENABLE_MAIN_TOOLBAR				0

#define OCHUSHA_DEFAULT_WIDTH				600
#define OCHUSHA_DEFAULT_HEIGHT				400
#define OCHUSHA_DEFAULT_BOARDLIST_WIDTH			150
#define OCHUSHA_DEFAULT_THREADLIST_HEIGHT		100

#define OCHUSHA_DEFAULT_BOARD_TAB_SHRINKABLE		FALSE
#define OCHUSHA_DEFAULT_BOARD_TAB_MINIMUM_SIZE		80
#define OCHUSHA_DEFAULT_BOARD_TAB_ENABLE_TOOLTIPS	FALSE

#define OCHUSHA_DEFAULT_THREAD_TAB_SHRINKABLE		TRUE
#define OCHUSHA_DEFAULT_THREAD_TAB_MINIMUM_SIZE		80
#define OCHUSHA_DEFAULT_THREAD_TAB_ENABLE_TOOLTIPS	TRUE


#define OCHUSHA_GTKRC				"ochusha-gtkrc"

#define THREAD_VIEW_EXTRA_WIDTH			30


#define OCHUSHA_NET_PROGRESS_ICON0		"Ochusha-Network-Progress0"
#define OCHUSHA_NET_PROGRESS_ICON1		"Ochusha-Network-Progress1"
#define OCHUSHA_NET_PROGRESS_ICON2		"Ochusha-Network-Progress2"
#define OCHUSHA_NET_PROGRESS_ICON3		"Ochusha-Network-Progress3"
#define OCHUSHA_NET_PREPARATION_ICON		"Ochusha-Network-Preparation"
#define OCHUSHA_NET_DONE_ICON			"Ochusha-Network-Done"
#define OCHUSHA_NET_ERROR_ICON			"Ochusha-Network-Error"

#define OCHUSHA_MINIMIZED_ICON			"Ochusha-Minimized"

#define TAB_LABEL_ICON_PROGRESS0		0
#define TAB_LABEL_ICON_PROGRESS1		1
#define TAB_LABEL_ICON_PROGRESS2		2
#define TAB_LABEL_ICON_PROGRESS3		3
#define TAB_LABEL_ICON_PREPARATION		4
#define TAB_LABEL_ICON_DONE			5
#define TAB_LABEL_ICON_ERROR			6


#ifdef __cplusplus
extern "C" {
#endif

typedef struct _OchushaApplication OchushaApplication;


#include "threadlist_filter.h"


struct _OchushaApplication
{
  GtkWindow *top_level;
  GtkBox *main_window;

  GtkWidget *menu_bar;

#if ENABLE_MAIN_TOOLBAR
  GtkToolbar *toolbar;

  /* toolbar */
  GtkCombo *url_box;
#endif

  GtkWidget *contents_window;		/* PanedNotebook */

  /* contents_windowselectorʬ */
  GtkWidget *boardlist_view;		/* BoardlistView */

  /* ߤĤΥ˥ե륿ŬѤ뤫ɤɽ */
  GtkToggleButton *filter_button;

  GtkStatusbar *statusbar;

  GtkClipboard *clipboard;


  /* İθǡ */
  BBSTable *bbs_table;


  /* windowʤ */
  gint width;
  gint height;
  gint boardlist_width;
  gint threadlist_height;


  /* paneɽˡʤ */
  gint boardlist_pane_style;
  gboolean board_tab_shrinkable;
  guint board_tab_minimum_size;
  gboolean board_tab_enable_tooltips;

  gint threadlist_pane_style;
  gboolean thread_tab_shrinkable;
  guint thread_tab_minimum_size;
  gboolean thread_tab_enable_tooltips;


  /* 쥿Υݥåץå״Ϣ */
  gboolean enable_popup_title;
  guint popup_title_delay;

  
  /* 쥹Υݥåץå״Ϣ */
  gboolean enable_popup_response;
  guint popup_response_delay;
  gboolean enable_smart_popup_placement;


  /* ֥֥饦Υޥɥ饤ƥץ졼 */
  char *web_browser_template;


  /* Υ桼ǥեȥե륿롼 */
  ThreadlistFilter user_default_filter;

  OchushaConfig conf;
};


typedef enum
{
  MAIN_MENU_QUIT = 1,
  MAIN_MENU_OPEN_URL,
  MAIN_MENU_REFRESH_BOARDLIST,
  MAIN_MENU_REFRESH_THREAD,
  MAIN_MENU_REFRESH_THREAD_ALL,
  MAIN_MENU_REFRESH_THREADLIST,
  MAIN_MENU_REFRESH_THREADLIST_ALL,
  MAIN_MENU_ABOUT,
} OchushaMainMenuAction;


#ifndef OCHUSHA_PRIVATE
extern OchushaNetworkCallbacks icon_indicator_funcs;
#endif


void ochusha_init_application(OchushaApplication *application);
void ochusha_set_main_menu_sensitive(OchushaMainMenuAction action,
				     gboolean sensitive);

void ochusha_clipboard_set_text(OchushaApplication *application,
				const gchar *text);

void ochusha_open_url(OchushaApplication *application, const gchar *url,
		      gboolean in_tab, gboolean use_web_browser);
#if ENABLE_MAIN_TOOLBAR
void set_current_url(OchushaApplication *application, const gchar *url);
const gchar *get_current_url(OchushaApplication *application);
#endif

GtkWidget *get_tab_label(const gchar *label_text);
void recycle_tab_label(GtkWidget *tab_label);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _OCHUSHA_UI_H_ */
